@@ -70,7 +70,7 @@
 #'     `index` for `is.pconsecutive` on data frames, for
 #'     further details see [pdata.frame()],
 #' @param id,time only relevant for default method: vectors specifying
-#'     the id and time dimensions, i. e. a sequence of individual and
+#'     the id and time dimensions, i. e., a sequence of individual and
 #'     time identifiers, each as stacked time series,
 #' @param \dots further arguments.
 #' @return A named `logical` vector (names are those of the
@@ -162,17 +162,16 @@ is.pconsecutive <- function(x, ...){
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
-  # argument 'x' just used for input check (if it is not NULL and is a vector)
-  
+  # argument 'x' just used for input check (if it is not NULL and is atomic)
+
   # input checks
-  if (length(id) != length(time)) 
+  if(length(id) != length(time)) 
     stop(paste0("arguments 'id' and 'time' must have same length: length(id): ", length(id), ", length(time) ", length(time)))
   
-  if (!is.null(x) && is.vector(x)) {
-    if (!(length(x) == length(id) && length(x) == length(time) && length(id) == length(time)))
+  if(!is.null(x) && is.atomic(x)) { # is.atomic was once is.vector, but is.vector is too strict as a factor is not a vector
+    if(!(length(x) == length(id) && length(x) == length(time) && length(id) == length(time)))
       stop(paste0("arguments 'x', 'id', 'time' must have same length: length(x): ", 
                   length(x), ", length(id): ", length(id), ", length(time): ", length(time)))
-    
   }
   
   # NB: 'time' is assumed to be organised as stacked time series (sorted for each individual)
@@ -182,7 +181,7 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
   #   is.pconsecutive.pdata.frame or is.pconsecutive.pseries as a pdata.frame (which is sorted) was constructed 
   #   in the first place; for data.frame interface the ordering is done in the respective function
   
-  if (na.rm.tindex) {
+  if(na.rm.tindex) {
     NA_tindex <- is.na(time)
     time <- time[!NA_tindex]
     id <- id[!NA_tindex]
@@ -193,16 +192,16 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
   #  see R FAQ 7.10 for coercing factors to numeric: 
   #      as.numeric(levels(factor_var))[as.integer(factor_var)]   is more efficient than
   #      as.numeric(as.character(factor_var))
-  if (!is.numeric(time) && is.factor(time)) time <- as.numeric(levels(time))[as.integer(time)]
+  if(!is.numeric(time) && is.factor(time)) time <- as.numeric(levels(time))[as.integer(time)]
   
   list_id_timevar <- split(time, id, drop = TRUE)
 
-  res <- sapply(list_id_timevar, function(id_timevar) { if(anyNA(id_timevar)) {
+  res <- vapply(list_id_timevar, function(id_timevar) { if(anyNA(id_timevar)) {
                                                            NA # return NA if NA found in the time periods for individual
                                                           } else {
                                                               begin <- id_timevar[1L]
                                                               end   <- id_timevar[length(id_timevar)]
-                                                 
+                                                              
                                                               # compare to length(original id_timevar) to find out if times are consecutive
                                                               (end - begin + 1L) == length(id_timevar)
                                                               
@@ -211,7 +210,8 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
                                                                 # consecutive <- seq(from = begin, to = end, by = 1)
                                                                 # length(consecutive) == length(id_timevar)
                                                           }
-                                                      })
+                                                      }, FUN.VALUE = TRUE)
+  
   return(res)
 }
 
@@ -222,8 +222,8 @@ is.pconsecutive.data.frame <- function(x, index = NULL, na.rm.tindex = FALSE, ..
     stop("if argument 'index' is not NULL, 'index' needs to specify
          'individual' and 'time' dimension for is.pconsecutive to work on a data.frame")
   
-  if (is.null(index)) index_orig_names <- names(x)[1:2] # assume first two columns to be the index vars
-    else index_orig_names <- index
+  # if index not provided, assume first two columns to be the index vars
+  index_orig_names <- if(is.null(index)) names(x)[1:2] else index
   
   id   <- x[ , index_orig_names[1L]]
   time <- x[ , index_orig_names[2L]]
@@ -243,29 +243,24 @@ is.pconsecutive.data.frame <- function(x, index = NULL, na.rm.tindex = FALSE, ..
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.pseries <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x, "index")
-  id   <- index[[1L]]
-  time <- index[[2L]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pconsecutive.default(x, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.pdata.frame <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x, "index")
-  id   <- index[[1L]]
-  time <- index[[2L]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pconsecutive.default(x, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.panelmodel <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x$model, "index")
-  id   <- index[[1L]]
-  time <- index[[2L]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x$model, "index")) # unclass for speed
+  # can determine solely based on indexes:
+  return(is.pconsecutive.default(NULL, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 
@@ -340,16 +335,12 @@ is.pbalanced <- function(x, ...) {
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.default <- function(x, y, ...) {
-  if (length(x) != length(y)) stop("The length of the two vectors differs\n")
+  if (length(x) != length(y)) stop("The length of the two inputs differs\n")
   x <- x[drop = TRUE] # drop unused factor levels so that table 
   y <- y[drop = TRUE] # gives only needed combinations
   z <- table(x, y)
-  if (any(v <- as.vector(z) == 0L)) {
-    balanced <- FALSE
-  } else { balanced <- TRUE }
-  
+  balanced <- if(any(v <- as.vector(z) == 0L)) FALSE else TRUE
   if (any(v > 1L)) warning("duplicate couples (id-time)\n")
-  
   return(balanced)
 }
 
@@ -357,36 +348,55 @@ is.pbalanced.default <- function(x, y, ...) {
 #' @export
 is.pbalanced.data.frame <- function(x, index = NULL, ...) {
   x <- pdata.frame(x, index)
-  index <- attr(x, "index")
-  id <- index[[1L]]
-  time <- index[[2L]]
-  return(is.pbalanced(id, time))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.pdata.frame <- function(x, ...) {
-  index <- attr(x, "index")
+  index <- unclass(attr(x, "index")) # unclass for speed
   return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.pseries <- function(x, ...) {
-  index <- attr(x, "index")
+  index <- unclass(attr(x, "index")) # unclass for speed
   return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pggls <- function(x, ...) {
+  # pggls is also class panelmodel, but take advantage of its pdim attribute
+  return(attr(x, "pdim")$balanced)
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pcce <- function(x, ...) {
+  # pcce is also class panelmodel, but take advantage of its pdim attribute
+  return(attr(x, "pdim")$balanced)
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pmg <- function(x, ...) {
+  # pmg is also class panelmodel, but take advantage of its pdim attribute
+  return(attr(x, "pdim")$balanced)
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pgmm <- function(x, ...) {
+  # pgmm is also class panelmodel, but take advantage of its pdim attribute
+  return(attr(x, "pdim")$balanced)
+}
+
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.panelmodel <- function(x, ...) {
   x <- model.frame(x)
   return(is.pbalanced(x))
 }
-
-#' @rdname is.pbalanced
-#' @export
-is.pbalanced.pgmm <- function(x, ...) {
-## pgmm is also class panelmodel, but take advantage of the pdim attribute in it
-  return(attr(x, "pdim")$balanced)
-}
