@@ -1,22 +1,36 @@
-## Function that are used in more that on place in plm. 
+## Function that are used in more than on place in plm (or likely to be used in more than one place in the future)
 
+## - trace : calculate trace of a matrix (used in ercomp())
+## - is.constant : check if a numeric vector or columns of a matrix is constant
 ## - bdiag : takes matrices as argument and returns the block-diagonal matrix (used in pgmm and plm.list)
+## - mylm : inner fitting func based on stats::lm with matrix inputs (used in plm.fit)
+## - my.lm.fit : like the barebone stats::lm.fit but with some extra information (e.g., SEs, sigma) used in purtest
 ## - twosls : computes the 2SLS estimator (used in plm and ercomp)
+## - data.name : used in a lot tests to generate the 'data.name' entry for htest objects from the model object's call$formula
 ## - has.intercept : tests the presence of an intercept
 ## - pres : extract model residuals as pseries (used in several estimation functions)
+## - punbalancedness : measures for the unbalancedness of panel data
+## - myvar : calculates variance with NA removal, checks if input is constant (also for factor and character)
+## - pvar : checks if input varies in individual / time dimension
+## - make.dummies : create a contrast-coded dummy matrix from a factor
+
+trace <- function(x) sum(diag(x))
+
+is.constant <- function(x) (max(x) - min(x)) < sqrt(.Machine$double.eps)
 
 bdiag <- function(...){
+  ## non-exported
   if (nargs() == 1L)
     x <- as.list(...)
   else
     x <- list(...)
   n <- length(x)
-  if(n == 0) return(NULL)
+  if(n == 0L) return(NULL)
   x <- lapply(x, function(y) if(length(y)) as.matrix(y) else
               stop("Zero-length component in x"))
   d <- array(unlist(lapply(x, dim)), c(2, n))
-  rr <- d[1, ]
-  cc <- d[2, ]
+  rr <- d[1L, ]
+  cc <- d[2L, ]
   rsum <- sum(rr)
   csum <- sum(cc)
   out <- array(0, c(rsum, csum))
@@ -29,65 +43,106 @@ bdiag <- function(...){
   ind[4,   ] <- ccum
   imat <- array(1:(rsum * csum), c(rsum, csum))
   iuse <- apply(ind, 2, function(y, imat) imat[(y[1L]+1):y[2L],
-                                               (y[3L]+1):y[4L]], imat=imat)
+                                               (y[3L]+1):y[4L]], imat = imat)
   iuse <- as.vector(unlist(iuse))
   out[iuse] <- unlist(x)
   return(out)
-} 
+}
 
+# mylm is used in plm.fit()
+mylm <- function(y, X, W = NULL) {
+  ## non-exported
+  names.X <- colnames(X)
+  result <- if(is.null(W)) lm(y ~ X - 1) else twosls(y, X, W)
+  if(any(na.coef <- is.na(result$coefficients))) {
+    ## for debug purpose:
+    # warning("Coefficient(s) '", paste((names.X)[na.coef], collapse = ", "), 
+    #"' could not be estimated and is (are) dropped.")
+    X <- X[ , !na.coef, drop = FALSE]
+    if(dim(X)[2L] == 0L) stop(paste("estimation not possible: all coefficients",
+                                    "omitted from estimation due to aliasing"))
+    
+    ## re-estimate without the columns which resulted previously in NA-coefficients
+    result <- if(is.null(W)) lm(y ~ X - 1) else twosls(y, X, W)
+  }
+  result$vcov <- vcov(result)
+  result$X <- X
+  result$y <- y
+  result$W <- W
+  # aliased is an element of summary.lm-objects:
+  # since plm drops aliased coefs, store this info in plm object
+  # NB: this only sets coefs to NA that are detected/set to NA by mylm()/lm.fit();
+  #     covariates dropped earlier by model.matrix( , cstcovar.rm) are not included here anymore
+  result$aliased <- na.coef
+  names(result$aliased) <- names.X
+  names(result$coefficients) <- colnames(result$vcov) <- 
+    rownames(result$vcov) <- colnames(X)
+  result
+}
 
-twosls <- function(y, X, W, intercept = FALSE){
-  Xhat <- lm(X ~ W)$fitted.values
-  if(!is.matrix(Xhat)){
+# my.lm.fit is used in purtest()
+my.lm.fit <- function(X, y, dfcor = TRUE, ...){
+  reg <- lm.fit(X, y)
+  ## 'as' summary method for lm.fit
+  p <- reg$rank
+  Qr <- reg$qr
+  n <- NROW(Qr$qr)
+  rdf <- n - p
+  p1 <- 1L:p
+  r <- reg$residuals
+  rss <- as.numeric(crossprod(r))
+  resvar <- if (dfcor) rss/rdf else rss/n
+  sigma <- sqrt(resvar)
+  R <- chol2inv(Qr$qr[p1, p1, drop = FALSE])
+  thecoef <- reg$coefficients[Qr$pivot[p1]] #[lags+1]
+  these <- sigma * sqrt(diag(R)) #[lags+1])
+  list(coef = thecoef, se = these, sigma = sigma,
+       rss = rss, n = n, K = p, rdf = rdf)
+}
+
+#' @importFrom stats .lm.fit
+twosls <- function(y, X, W, intercept = FALSE, lm.type = "lm"){
+  ## non-exported
+  # Return value can be controlled by argument lm.type. Often, a full lm model
+  # is needed for further processing but can select one of the fast but less
+  # rich objects produced by lm.fit or .lm.fit (the latter does not contain, e.g.,
+  # fitted.values and is to be used very carefully (e.g., coefs not in input order)).
+
+  # As NA/NaN/(+/-)Inf-freeness needs to be guaranteed when functions call
+  # twosls(), so can use lm.fit to calc. Xhat.
+  Xhat <- lm.fit(cbind(1, W), X)$fitted.values
+  # old: Xhat <- lm(X ~ W)$fitted.values
+  
+  if(!is.matrix(Xhat)) {
+    # ensure Xhat is a matrix
     Xhat <- matrix(Xhat, ncol = 1L)
     colnames(Xhat) <- colnames(X)
   }
-  if(intercept){
-    model <- lm(y ~ Xhat)
+  
+  if(intercept) {
+    model <- switch(lm.type,
+                    "lm"      =  lm(y ~ Xhat),
+                    "lm.fit"  =  lm.fit(cbind(1, Xhat), y),
+                    ".lm.fit" = .lm.fit(cbind(1, Xhat), y))
     yhat <- as.vector(crossprod(t(cbind(1, X)), coef(model)))
   }
   else{
-    model <- lm(y ~ Xhat - 1)
+    model <- switch(lm.type,
+                    "lm"      =  lm(y ~ Xhat - 1),
+                    "lm.fit"  =  lm.fit(Xhat, y),
+                    ".lm.fit" = .lm.fit(Xhat, y))
     yhat <- as.vector(crossprod(t(X), coef(model)))
   }
   model$residuals <- y - yhat
   model
 }
 
-## expand.formula <- function(x){
-##   oclass <- class(x)
-##   if (! any(class(x) == "Formula")) stop("not a Formula object")
-##   if (length(x)[2] != 2) stop("not a two part formula")
-##   xs <- structure(x, class = "formula")
-##   has.response <- attr(terms(xs),"response") == 1
-##   if (has.response){
-##     y <- x[[2]]
-##     rhs <- x[[3]]
-##   }
-##   else{
-##     y <- NULL
-##     rhs <- x[[2]]
-##   }
-##   firstpart <- rhs[[2]]
-##   secondpart <- rhs[[3]]
-##   if (has.response){
-##     one <- do.call("~", list(y,firstpart))
-##     two <- do.call("~", list(y,secondpart))
-##   }
-##   else{
-##     one <- do.call("~", list(firstpart))
-##     two <- do.call("~", list(secondpart))
-##   }
-##   two <- update(one, two)
-##   one <- paste(deparse(one), collapse = "")
-##   two <- paste(deparse(two[[3]]), collapse = "")
-##   result <- as.formula(paste(one, "|", two, collapse = ""));
-##   result <- as.Formula(result)
-##   #YC  class(result) <- c("pFormula", class(result))
-##   structure(result, class = oclass)
-## }
-
-
+data.name <- function(x) {
+  ## non-exported, used in various tests
+  data.name <- paste(deparse(x$call$formula))
+  if (length(data.name) > 1L) paste(data.name[1L], "...")
+  else data.name
+}
 
 ##### has.intercept methods #####
 
@@ -96,16 +151,17 @@ twosls <- function(y, X, W, intercept = FALSE){
 #'
 #' The presence of an intercept is checked using the formula which is
 #' either provided as the argument of the function or extracted from
-#' a fitted model
+#' a fitted model.
 #'
 #' @param object a `formula`, a `Formula` or a fitted model (of class
 #'     `plm` or `panelmodel`),
-#' @param rhs,part the index of the right hand sides part of the
-#'     formula for which one wants to check the presence of an
-#'     intercept (relevant for the `Formula` and the `plm` methods),
+#' @param rhs an integer (length > 1 is possible), indicating the parts of right
+#'      hand sides of the formula to be evaluated for the presence of an
+#'      intercept or NULL for all parts of the right hand side
+#'      (relevant for the `Formula` and the `plm` methods)
 #' @param \dots further arguments.
 #'
-#' @return a boolean
+#' @return a logical
 #' @export
 has.intercept <- function(object, ...) {
   UseMethod("has.intercept")
@@ -128,14 +184,14 @@ has.intercept.formula <- function(object, ...) {
 has.intercept.Formula <- function(object, rhs = NULL, ...) {
   ## NOTE: returns a logical vector of the necessary length
   ## (which might be > 1)
-    if(is.null(rhs)) rhs <- 1:length(attr(object, "rhs"))
-    sapply(rhs, function(x){
-        aform <- formula(object, lhs = 0, rhs = x)
-        # expand the dot if any in all the parts except the first
-        if(x > 1) aform <- update(formula(object, lhs = 0, rhs = 1), aform)
-        has.intercept(aform)
-    }
-    )
+  if (is.null(rhs)) rhs <- 1:length(attr(object, "rhs"))
+  res <- sapply(rhs, function(x) {
+    aform <- formula(object, lhs = 0, rhs = x)
+    # expand the dot if any in all the parts except the first
+    if (x > 1L) aform <- update(formula(object, lhs = 0, rhs = 1), aform)
+    has.intercept(aform)
+  })
+  return(res)
 }
 
 #' @rdname has.intercept
@@ -147,25 +203,27 @@ has.intercept.panelmodel <- function(object, ...) {
 
 #' @rdname has.intercept
 #' @export
-has.intercept.plm <- function(object, part = "first", ...) {
-  has.intercept(formula(object), part = part)
+has.intercept.plm <- function(object, rhs = 1L, ...) {
+  has.intercept(formula(object), rhs = rhs, ...)
 }
 
+
 pres <- function(x) {  # pres.panelmodel
   ## extracts model residuals as pseries
   ## not necessary for plm models as residuals.plm returns a pseries,
   ## but used in residuals.pggls, residuals.pcce, residuals.pmg
   
   ## extract indices
-  groupind <-attr(x$model, "index")[ , 1L]
-  timeind  <-attr(x$model, "index")[ , 2L]
+  xindex <- unclass(attr(x$model, "index")) # unclass for speed
+  groupind <- xindex[[1L]]
+  timeind  <- xindex[[2L]]
   
   # fix to allow operation with pggls, pmg
   # [TODO: one day, make this cleaner; with the describe framework?]
   if (!is.null(x$args$model))                 maybe_fd <- x$args$model
   if (!is.null(attr(x, "pmodel")$model.name)) maybe_fd <- attr(x, "pmodel")$model.name # this line is currently needed to detect pggls models
   
-  ## Achim's fix: reduce id and time index to accomodate first-differences model's number of observations
+  ## Achim's fix: reduce id and time index to accommodate first-differences model's number of observations
   if(exists("maybe_fd") && maybe_fd == "fd") {
     groupi <- as.numeric(groupind)
     ## make vector =1 on first obs in each group, 0 elsewhere
@@ -183,10 +241,15 @@ pres <- function(x) {  # pres.panelmodel
 }
 
 
-# punbalancedness: measures for unbalancedness of a pandel data set as
-# defined in \insertCite{AHRE:PINC:81;textual}{plm}, p. 228 (gamma and
-# nu) and for nested panel structures as in
-# \insertCite{BALT:SONG:JUNG:01;textual}{plm}, pp. 368-369
+# punbalancedness: measures for unbalancedness of a panel data set as
+# defined in Ahrens/Pincus (1981), p. 228 (gamma and
+# nu) and for nested panel structures as in Baltagi/Song/Jung (2001), pp. 368-369 .
+#
+# Ahrens/Pincus (1981), On Two Measures of Unbalancedness in a One-Way Model
+#  and Their Relation to Efficiency, Biometrical Journal, Vol. 23, pp. 227-235.
+# 
+# Baltagi/Song/Jung (2001), The unbalanced nested error component regression model,
+#  Journal of Econometrics, Vol. 101, pp. 357-381
 
 
 #' Measures for Unbalancedness of Panel Data
@@ -257,7 +320,7 @@ pres <- function(x) {  # pres.panelmodel
 #'     number of observations (individual, time) actually used for
 #'     model estimation are taken into account. When called on a
 #'     `(p)data.frame`, the rows in the `(p)data.frame` are
-#'     considered, disregarding any NA values in the dependent or
+#'     considered, disregarding any `NA` values in the dependent or
 #'     independent variable(s) which would be dropped during model
 #'     estimation.
 #' @export
@@ -316,6 +379,7 @@ punbalancedness <- function(x, ...) {
 punbalancedness.default <- function(x, ...) {
 
   ii <- index(x)
+  if(!is.index(ii)) stop("no valid index found for input object 'x'")
   
   if (ncol(ii) == 2L) {
    ## original Ahrens/Pincus (1981)
@@ -332,6 +396,7 @@ punbalancedness.default <- function(x, ...) {
     if (ncol(ii) == 3L) {
      ## extension to nested model with additional group variable
      ## Baltagi/Song/Jung (2001), pp. 368-369
+      ii <- unclass(ii) # unclass for speed
       ids <- ii[[1L]]
       tss <- ii[[2L]]
       gps <- ii[[3L]]
@@ -374,6 +439,7 @@ punbalancedness.panelmodel <- function(x, ...) {
 
 
 myvar <- function(x){
+  ## non-exported
   x.na <- is.na(x)
   if(anyNA(x.na)) x <- x[!x.na]
   n <- length(x)
@@ -395,7 +461,7 @@ myvar <- function(x){
 #' This function checks for each variable of a panel if it varies
 #' cross-sectionally and over time.
 #' 
-#' For (p)data.frame and matrix interface: All-NA columns are removed
+#' For (p)data.frame and matrix interface: All-`NA` columns are removed
 #' prior to calculation of variation due to coercing to pdata.frame
 #' first.
 #' 
@@ -414,15 +480,16 @@ myvar <- function(x){
 #'
 #' \item{id.variation_anyNA}{a logical vector with `TRUE` values if
 #' the variable has at least one individual-time combination with all
-#' NA values in the individual dimension for at least one time period,
+#' `NA` values in the individual dimension for at least one time period,
 #' `FALSE` if not,}
 #'
 #' \item{time.variation_anyNA}{a logical vector with `TRUE` values if
 #' the variable has at least one individual-time combination with all
-#' NA values in the time dimension for at least one individual,
+#' `NA` values in the time dimension for at least one individual,
 #' `FALSE` if not.}
 #' 
-#' @note `pvar` can be time consuming for ``big'' panels.
+#' @note `pvar` can be time consuming for ``big'' panels. As a fast alternative
+#' [collapse::varying()] from package \CRANpkg{collapse} could be used. 
 #' @export
 #' @author Yves Croissant
 #' @seealso [pdim()] to check the dimensions of a 'pdata.frame' (and
@@ -461,7 +528,7 @@ pvar.default <- function(x, id, time, ...){
   id.variation   <- rep(TRUE, len)
   time.variation_anyNA <- rep(FALSE, len)
   id.variation_anyNA   <- rep(FALSE, len)
-  lid <- split(x, id)   # these split() functions seem particularly slow
+  lid   <- split(x, id)   # these split() functions seem particularly slow
   ltime <- split(x, time)
   if(is.list(x)){
     if(len == 1L){
@@ -544,10 +611,8 @@ pvar.data.frame <- function(x, index = NULL, ...){
 #' @rdname pvar
 #' @export
 pvar.pdata.frame <- function(x, ...){
-  index <- attr(x, "index")
-  id <- index[[1L]]
-  time <- index[[2L]]
-  pvar.default(x, id, time)
+  index <- unclass(attr(x, "index")) # unclass for speed
+  pvar.default(x, index[[1L]], index[[2L]])
 }
 
 #' @rdname pvar
@@ -566,7 +631,7 @@ print.pvar <- function(x, ...){
   if(any(!x$time.variation)){
     var <- varnames[x$time.variation == FALSE]
     #    if (!is.null(y)) var <- var[-which(var==y$id)]
-    if (length(var)!=0) cat(paste("no time variation:      ", paste(var,collapse=" "),"\n"))
+    if(length(var)!=0) cat(paste("no time variation:      ", paste(var,collapse=" "),"\n"))
   }
   if(any(!x$id.variation)){
     var <- varnames[x$id.variation == FALSE]
@@ -577,39 +642,148 @@ print.pvar <- function(x, ...){
   # any individual-time combinations all NA?
   if(any(x$time.variation_anyNA)){
     var_anyNA <- varnames[x$time.variation_anyNA]
-    if (length(var_anyNA)!=0) cat(paste("all NA in time dimension for at least one individual: ", paste(var_anyNA,collapse=" "),"\n"))
+    if(length(var_anyNA)!=0) cat(paste("all NA in time dimension for at least one individual: ", paste(var_anyNA,collapse=" "),"\n"))
   }
   if(any(x$id.variation_anyNA)){
     var_anyNA <- varnames[x$id.variation_anyNA]
-    if (length(var_anyNA)!=0) cat(paste("all NA in ind. dimension for at least one time period:", paste(var_anyNA,collapse=" "),"\n"))
+    if(length(var_anyNA)!=0) cat(paste("all NA in ind. dimension for at least one time period:", paste(var_anyNA,collapse=" "),"\n"))
   }
+  invisible(x)
 }
 
-## Non-exported internal function for subsetting of pseries. Can be used
-## internally. 
-## Currently we do not have "proper" subsetting function for pseries
-## ([.pseries) which applies in any case (but see the sketch to be tested in
-##  tool_pdata.frame.R)
-subset_pseries <- function(x, ...) {
 
- ## use '...' instead of only one specific argument, because subsetting for
- ## factors can have argument 'drop', e.g., x[i, drop=TRUE] see ?Extract.factor
-  index <- attr(x, "index")
-  if (is.null(index)) warning("pseries object with is.null(index(pseries)) == TRUE encountered, trying to continue anyway...")
-  if (!is.index(index)) stop(paste0("pseries object has illegal index with class(index) == ", paste0(class(index), collapse = ", ")))
-  names_orig <- names(x)
-  x <- remove_pseries_features(x)
-  result <- x[...]
-
-  # subset index / identify rows to keep in the index:
-  keep_rownr <- seq_along(names_orig)  # full length row numbers original pseries
-  names(keep_rownr) <- names_orig
-  keep_rownr <- keep_rownr[names(result)] # row numbers to keep after subsetting
-  index <- index[keep_rownr, ]
-
-  # drop unused levels (like in subsetting of pdata.frames)
-  index <- droplevels(index)
-
-  result <- add_pseries_features(result, index)
-  return(result)
+#' Create a Dummy Matrix
+#' 
+#' Contrast-coded dummy matrix created from a factor
+#' 
+#' This function creates a matrix of dummies from the levels of a factor.
+#' In model estimations, it is usually preferable to not create the dummy matrix 
+#' prior to estimation but to simply specify a factor in the formula and let the
+#' estimation function handle the creation of the dummies.
+#' 
+#' This function is merely a convenience wrapper around `stats::contr.treatment`
+#' to ease the dummy matrix creation process shall the dummy matrix be explicitly
+#' required. See Examples for a use case in LSDV (least squares dummy variable)
+#' model estimation.
+#' 
+#' The default method uses a factor as main input (or something coercible to a 
+#' factor) to derive the dummy matrix from. Methods for data frame and pdata.frame 
+#' are available as well and have the additional argument `col` to specify the
+#' the column from which the dummies are created; both methods merge the dummy 
+#' matrix to the data frame/pdata.frame yielding a ready-to-use data set.
+#' See also Examples for use cases.
+#' 
+#' @param x a factor from which the dummies are created (x is coerced to 
+#'          factor if not yet a factor) for the default method or a data 
+#'          data frame/pdata.frame for the respective method.
+#' @param base integer or character, specifies the reference level (base), if 
+#'             integer it refers to position in `levels(x)`, if character the name 
+#'             of a level,
+#' @param base.add logical, if `TRUE` the reference level (base) is added 
+#'                 to the return value as first column, if `FALSE` the reference
+#'                 level is not included.
+#' @param col character (only for the data frame and pdata.frame methods), to
+#'            specify the column which is used to derive the dummies from,
+#' @param \dots further arguments.
+#'
+#' @return For the default method, a matrix containing the contrast-coded dummies, 
+#'         dimensions are n x n where `n = length(levels(x))` if argument  
+#'        `base.add = TRUE` or `n = length(levels(x)-1)` if `base.add = FALSE`;
+#'         for the data frame and pdata.frame method, a data frame or pdata.frame,
+#'         respectively, with the dummies appropriately merged to the input as 
+#'         last columns (column names are derived from the name of the column 
+#'         used to create the dummies and its levels).
+#' @author Kevin Tappe
+#' @importFrom stats contr.treatment
+#' @export
+#' @seealso [stats::contr.treatment()], [stats::contrasts()]
+#' @keywords manip
+#' @examples
+#' library(plm)
+#' data("Grunfeld", package = "plm")
+#' Grunfeld <- Grunfeld[1:100, ] # reduce data set (down to 5 firms)
+#'
+#' ## default method
+#' make.dummies(Grunfeld$firm) # gives 5 x 5 matrix (5 firms, base level incl.)
+#' make.dummies(Grunfeld$firm, base = 2L, base.add = FALSE) # gives 5 x 4 matrix
+#'
+#' ## data frame method
+#' Grun.dummies <- make.dummies(Grunfeld, col = "firm")
+#' 
+#' ## pdata.frame method
+#' pGrun <- pdata.frame(Grunfeld)
+#' pGrun.dummies <- make.dummies(pGrun, col = "firm")
+#' 
+#' ## Model estimation:
+#' ## estimate within model (individual/firm effects) and LSDV models (firm dummies)
+#' # within model:
+#' plm(inv ~ value + capital, data = pGrun, model = "within")
+#' 
+#' ## LSDV with user-created dummies by make.dummies:
+#' form_dummies <- paste0("firm", c(1:5), collapse = "+")
+#' form_dummies <- formula(paste0("inv ~ value + capital + ", form_dummies))
+#' plm(form_dummies, data = pGrun.dummies, model = "pooling") # last dummy is dropped
+#'
+#' # LSDV via factor(year) -> let estimation function generate dummies:
+#' plm(inv ~ value + capital + factor(firm), data = pGrun, model = "pooling")
+make.dummies <- function(x, ...){
+  UseMethod("make.dummies")
 }
+
+#' @rdname make.dummies
+#' @export
+make.dummies.default <- function(x, base = 1L, base.add = TRUE, ...) {
+  
+  stopifnot(is.numeric(base) || is.character(base))
+  if(is.numeric(base)) if(round(base) != base) stop("Argument 'ref' specified as numeric but is not integer")
+  if(!is.factor(x)) x <- factor(x)
+  
+  lvl <- levels(x)
+  
+  if(is.character(base)) {
+    pos <- match(base, lvl)
+    if(is.na(pos)) stop(paste0("argument 'ref' specified as character but value \"", 
+                               base, "\", is not in levels(x)"))
+    base <- pos
+  }
+  
+  dummies <- contr.treatment(lvl, base = base)
+  
+  # if requested, add reference level to dummy matrix in 1st position
+  if(base.add) {
+    lvl.base <- levels(x)[base]
+    dummies <- cbind(c(1, rep(0, NROW(dummies)-1)), dummies)
+    colnames(dummies) <- c(lvl.base, colnames(dummies)[-1L])  
+  }
+  dummies # is a matrix
+}
+
+#' @rdname make.dummies
+#' @export
+make.dummies.data.frame <- function(x, col, base = 1L, base.add = TRUE, ...) {
+
+  stopifnot(inherits(col, "character"))
+  dum.mat <- make.dummies.default(x[ , col], base, base.add) # dummy matrix
+  colnames(dum.mat) <- paste0(col, colnames(dum.mat))
+  dum.df <- data.frame(cbind("merge.col" = rownames(dum.mat), dum.mat))
+ 
+  merge(x, dum.df, by.x = col, by.y = "merge.col", sort = FALSE)
+}
+
+#' @rdname make.dummies
+#' @export
+make.dummies.pdata.frame <- function(x, col, base = 1L, base.add = TRUE, ...) {
+  
+  stopifnot(inherits(col, "character"))
+#  idx.pos <- pos.index(x)
+#  drop.idx <- anyNA(idx.pos)
+  idx <- attr(x, "index")
+  res <- make.dummies.data.frame(x, col, base, base.add)
+  # add back pdata.frame features (assumption is: merge did not change order of original data.frame)
+  attr(res, "index") <- idx
+  class(res) <- c("pdata.frame", class(res))
+  res
+}
+
+
+
